/* $Id: housekeep.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            housekeep.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Various housekeeping functions, such as initiation and
 *                  cleaning up.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>

#ifdef WIN32
#include <winsock.h>
#endif

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snInit
 *
 *  FUNCTION      Initiate the library.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr snInit(void);
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Do necessary setup for the use of this library. This
 *                function _must_ be called before any of the others
 *                (except snSetErrorHandling) in the library are called.
 */
SNErr
snInit(void)
{
#ifdef WIN32
    /* NOTE: this is neither tested, nor supported at the moment. */

    /* initialize socketlib */
    WORD    wVersionRequested;
    WSADATA wsaData;

    wVersionRequested = MAKEWORD(1, 1);
    if (WSAStartup(wVersionRequested, &wsaData) != 0) {
	/* unable to initialize winsock */
	/* TODO: return something useful,
	 *       possibly refering to WSAGetLastError. */
    }
    if (LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1) {
	WSACleanup();
	/* wrong winsock version */
	/* TODO: return something useful,
	 *       possibly refering to WSAGetLastError. */
    }
#else
    /* do nothing for Unix. */
#endif

    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          snFinish
 *
 *  FUNCTION      Tidy up before exiting.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr snFinish(void);
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Do necessary cleanup after using the library.
 */
SNErr
snFinish(void)
{
#ifdef WIN32
    /* NOTE: this is neither tested, nor supported at the moment. */
    WSACleanup();
#else
    /* do nothing for Unix. */
#endif

    return _snError(SN_OK, NULL);
}
