/* $Id: midruns.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            midruns.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Machine Independent Data Representation, routines for
 *                  handling unsigned entities.
 *
 *                  See midrgen.c for more info.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include "shhmidr.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutByte
 *
 *  FUNCTION      Enter a byte in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutByte(MIDR *midr, BYTE x);
 *
 *  INPUT         midr    the structure in question.
 *                x       the byte to store.
 */
void
midrPutByte(MIDR *midr, BYTE x)
{
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx);
    *p = x;
    if ((midr->idx += sizeof(BYTE)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetByte
 *
 *  FUNCTION      Fetch a byte from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                BYTE midrGetByte(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next byte from the buffer.
 */
BYTE
midrGetByte(MIDR *midr)
{
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx);
    midr->idx += sizeof(BYTE);
    return *p;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutWord
 *
 *  FUNCTION      Enter a word in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutWord(MIDR *midr, WORD x);
 *
 *  INPUT         x       the word to store.
 *                midr    the structure in question.
 */
void
midrPutWord(MIDR *midr, WORD x)
{
    register int q;
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx + sizeof(WORD));
    for (q = sizeof(WORD); q; q--) {
	*--p = x & 0xFF;
	x >>= 8;
    }
    if ((midr->idx += sizeof(WORD)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetWord
 *
 *  FUNCTION      Fetch a word from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                WORD midrGetWord(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next word from the buffer.
 */
WORD
midrGetWord(MIDR *midr)
{
    register int q;
    BYTE *p;
    WORD ret = 0;

    p = (BYTE *) (midr->buff + midr->idx);
    for (q = sizeof(WORD); q; q--)
	ret = (ret << 8) | *p++;
    midr->idx += sizeof(WORD);
    return ret;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutDWord
 *
 *  FUNCTION      Enter a dword in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutDWord(MIDR *midr, DWORD x);
 *
 *  INPUT         midr    the structure in question.
 *                x       the dword to store.
 */
void
midrPutDWord(MIDR *midr, DWORD x)
{
    register int q;
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx + sizeof(DWORD));
    for (q = sizeof(DWORD); q; q--) {
	*--p = x & 0xFF;
	x >>= 8;
    }
    if ((midr->idx += sizeof(DWORD)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetDWord
 *
 *  FUNCTION      Fetch a dword from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                DWORD midrGetDWord(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next dword from the buffer.
 */
DWORD
midrGetDWord(MIDR *midr)
{
    register int q;
    BYTE *p;
    DWORD ret = 0;

    p = (BYTE *) (midr->buff + midr->idx);
    for (q = sizeof(DWORD); q; q--)
	ret = (ret << 8) | *p++;
    midr->idx += sizeof(DWORD);
    return ret;
}
