/* $Id: echo-client.c,v 1.2 2002/01/19 19:15:06 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            echo-client.c
 *
 *  DESCRIPTION     Sample echo client, to illustrate the use of the
 *                  shhnet library for creating a TCP client.
 *                  Will connect to an echo server on the local host,
 *                  send a couple of lines, and display the result.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <shhnet.h>

#define DEBUG

#define PROG_NAME "echo-client"
#define ECHO_PORT "9001"        /* echo-server sample program */
/* #define ECHO_PORT "echo" */  /* "real" echo server */

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          sendAndReceiveEcho
 *
 *  FUNCTION      Send a line, and receive the echo.
 *
 *  INPUT         sock    the socket referencing the open connection.
 *                line    text to send. the line must be ended with
 *                        "\r\n" (CR LF) to follow the telnet-convention.
 */
static void
sendAndReceiveEcho(int sock, const char *line)
{
    char received[200];

    /* display what to send, discarding the two line termination characters. */
    printf("sending `%.*s'\n", (int) (strlen(line) - 2), line);

    /* send line, and receive echo. */
    tcpPuts(sock, line);
    tcpGets(sock, received, sizeof(received));

    /* check if server is still with us, and display result. */
    if (strlen(received) == 0)
	printf("eh... the server didn't return anything!\n");
    else
	printf("server returned `%.*s'\n",
	       (int) (strlen(received) - 2), received);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(int argc, char *argv[])
{
    int sock;

    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
    snInit();

#ifdef DEBUG
    fprintf(stderr, "%s started\n", PROG_NAME);
#endif
    tcpClientOpen("localhost", ECHO_PORT, &sock);
    sendAndReceiveEcho(sock, "this line should be repeated.\r\n");
    sendAndReceiveEcho(sock, "and so should this line.\r\n");
    tcpClose(sock);
#ifdef DEBUG
    fprintf(stderr, "%s finished\n", PROG_NAME);
#endif

    snFinish();
    return 0;
}
