/* $Id: shhnet.h,v 0.15 2001/01/16 21:21:01 sverrehu Exp $ */
#ifndef SHHNET_H
#define SHHNET_H

/* belongs to the shhnet library. */

#include <sys/types.h>  /* size_t, fd_set */
#include <sys/time.h>   /* struct timeval */
#include <netinet/in.h> /* htonl, ntohl */

#ifdef __cplusplus
  extern "C" {
#endif

#ifdef __GNUC__
#define SN_FORMAT_1 __attribute__ ((format (printf, 1, 2)))
#define SN_FORMAT_2 __attribute__ ((format (printf, 2, 3)))
#else
#define SN_FORMAT_1
#define SN_FORMAT_2
#endif

typedef unsigned long  SNAddr;    /* an IP-address */
typedef unsigned short SNPort;    /* a port number */

typedef enum {
    SN_REPORT_ERROR_AND_EXIT,     /* display message, and abort program */
    SN_REPORT_ERROR_AND_RETURN,   /* display message, and return error code */
    SN_EXIT,                      /* be silent, and exit */
    SN_RETURN_ERROR               /* be silent, and return error code */
} SNErrorHandling;

typedef enum { 
    SN_OK = 0,                    /* no error */
    SN_ERR_INTERNAL,              /* internal error */
    SN_ERR_ERRNO,                 /* error code found in errno */
    SN_ERR_H_ERRNO,               /* error code found in h_errno */
    SN_ERR_UNKNOWN_SERVICE        /* unknown service in inServiceToPort */
    /* remember to update snErrToStr() in error.c if adding new codes */
} SNErr;

typedef enum {
    SN_PROTO_TCP,                 /* TCP protocol */
    SN_PROTO_UDP                  /* UDP protocol */
} SNProto;

SNErr snInit(void);               /* must be called at startup! */
SNErr snFinish(void);             /* should be called at exit */

void  snSetErrorHandling(SNErrorHandling type, void (*f)(const char *, ...));
const char *snErrToStr(SNErr err);
SNErr snGetLastError(void);

SNErr tcpClientOpen(const char *host, const char *service, int *sock);
SNErr tcpServerOpenAddr(SNAddr addr, const char *service, int *sock);
SNErr tcpServerOpen(const char *service, int *sock);
SNErr tcpClose(int sock);
SNErr tcpListen(int sock, int backlog);
SNErr tcpAccept(int sock, int *newsock);
SNErr tcpRead(int sock, char *buf, size_t count, size_t *nread);
SNErr tcpWrite(int sock, const char *buf, size_t count, size_t *nwritten);
SNErr tcpReadAll(int sock, char *buf, size_t count);
SNErr tcpWriteAll(int sock, const char *buf, size_t count);
SNErr tcpGets(int sock, char *s, int n);
SNErr tcpPuts(int sock, const char *s);
SNErr tcpPrintf(int sock, const char *format, ...) SN_FORMAT_2;

SNErr udpClientOpen(const char *host, const char *service, int *sock);
SNErr udpServerOpenAddr(SNAddr addr, const char *service, int *sock);
SNErr udpServerOpen(const char *service, int *sock);
SNErr udpMulticastOpen(const char *group, const char *port,
		       int ttl, int *sock);
SNErr udpClientSock(int *sock);
SNErr udpClose(int sock);
SNErr udpRead(int sock, char *buf, size_t count, size_t *nread);
SNErr udpWrite(int sock, const char *buf, size_t count, size_t *nwritten);
SNErr udpReadFrom(int sock, char *buf, size_t count,
		  SNAddr *addr, SNPort *port, size_t *nread);
SNErr udpWriteTo(int sock, const char *buf, size_t count,
		 SNAddr addr, SNPort port, size_t *nwritten);

SNErr sockCreate(SNProto proto, int *sock);
SNErr sockBind(int sock, SNAddr addr, SNPort port);
SNErr sockConnect(int sock, SNAddr addr, SNPort port);
SNErr sockSetNoDelay(int sock, int onoff);
SNErr sockSetBroadcast(int sock, int onoff);
SNErr sockSetKeepAlive(int sock, int onoff);
SNErr sockSetLinger(int sock, int onoff);
SNErr sockSetOutOfBandInline(int sock, int onoff);
SNErr sockSetReuseAddr(int sock, int onoff);
SNErr sockJoinMulticastGroup(int sock, SNAddr group);
SNErr sockLeaveMulticastGroup(int sock, SNAddr group);
SNErr sockSetMulticastTtl(int sock, int ttl);
SNErr sockSetMulticastLoop(int sock, int onoff);

#define inHostToNetworkAddr(x) ((SNAddr) (htonl((SNAddr) (x))))
#define inNetworkToHostAddr(x) ((SNAddr) (ntohl((SNAddr) (x))))
#define inHostToNetworkPort(x) ((SNPort) (htons((SNPort) (x))))
#define inNetworkToHostPort(x) ((SNPort) (ntohs((SNPort) (x))))

#define SN_ANY_ADDR (inHostToNetworkAddr(INADDR_ANY))
#define SN_ANY_PORT (inHostToNetworkPort((SNPort) 0))

SNErr inHostAddrToStr(SNAddr addr, char *s);
SNErr inHostAddrToName(SNAddr addr, char *s, int len);
SNErr inHostNameToAddr(const char *s, SNAddr *addr);
SNErr inServiceToPort(const char *s, const char *proto, SNPort *port);
SNErr inGetSockAddrPort(int sock, SNAddr *addr, SNPort *port);
SNErr inGetSockStr(int sock, char *s, int len, int lookupName);
SNErr inGetPeerAddrPort(int sock, SNAddr *addr, SNPort *port);
SNErr inGetPeerStr(int sock, char *s, int len, int lookupName);

typedef void (*SNSigHandler)(int);
SNErr snClientSetupSigHandlers(SNSigHandler termHandler);
SNErr snServerSetupSigHandlers(SNSigHandler termHandler,
			       SNSigHandler hupHandler);

SNErr snSelect(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
	       struct timeval *timeout);

int   snChopLf(char *s);
int   snChopCrLf(char *s);

#undef SN_FORMAT_1
#undef SN_FORMAT_2

#ifdef __cplusplus
  }
#endif

#endif
