/* $Id: error.c,v 1.2 2002/01/19 19:15:06 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            error.c
 *
 *  DESCRIPTION     Testing error functions.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdarg.h>

#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static void
errorPrintf(const char *format, ...)
{
    va_list ap;

    printf("tst shows you this error: ");
    va_start(ap, format);
    vfprintf(stdout, format, ap);
    va_end(ap);
}

static void
testError(void)
{
    SNErr err;
    SNPort  port;

    snSetErrorHandling(SN_RETURN_ERROR, NULL);
    printf("generating an error with no message from shhnet\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("i got the error code %d (%s) in return\n",
	   (int) err, snErrToStr(err));
    printf("\n");

    snSetErrorHandling(SN_REPORT_ERROR_AND_RETURN, NULL);
    printf("generating an error with default message from shhnet\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("i got the error code %d (%s) in return\n",
	   (int) err, snErrToStr(err));
    printf("\n");

    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, errorPrintf);
    printf("generating an error with my message while shhnet aborts\n");
    err = inServiceToPort("qwe", "tcp", &port);
    printf("whee! I'm still here!\n");
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(void)
{
    snInit();

    testError(); /* aborts the program */

    snFinish();
    return 0;
}
